/*------------------------------------------------------------------------------
* Alpha Shooter
* A 3D OpenGL First-Person Shooter game
* Copyright (C) 2005, 2006, 2007 Nicola Cocchiaro <ncocchiaro@users.sf.net>
* Alpha Shooter can be found at http://www.sf.net/projects/alphashooter.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 or, (at your option) any later
* version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*-------------------------------------------------------------------------------
*/


// defs.h
// helper definitions

#ifndef ALPSHOOT_DEFS_H
#define ALPSHOOT_DEFS_H

#ifdef ALPSHOOT_WIN32
  #include <windows.h>
  #include <GL/gl.h>
  #include <GL/glext.h>
  #include <GL/glu.h>
  #include <GL/glut.h>
#else
  #ifdef ALPSHOOT_MACOSX
    #include <gl.h>
    #include <glu.h>
    #include <glut.h>
  #else
    #include <GL/gl.h>
    #include <GL/glu.h>
    #include <GL/glut.h>
  #endif  /* ALPSHOOT_MACOSX */
#endif  /* ALPSHOOT_WIN32 */

#include <math.h>
#include <string.h>

/* useful macros */
#define DSGN(x) x >= 0.0 ? 1.0 : -1.0
#define DABS(x) x >= 0.0 ? x : ((-1.0)*x)
#define DBGF(s,p) printf("%s %f\n", s, p)
#define MALLOC_CHECK(a) {if((a)==NULL){fprintf(stderr,"%s %d: malloc failed\n",__FILE__,__LINE__);abort();}}
/* arbitrary precision for comparison between floating point values */
#define F_PRECISION	0.0000000000005

/* steps for translation and rotation (degrees) */
#define STEP 0.05
#define ROTSTEP 0.35

/* number of possible screen resolutions */
#define MAX_RES			3
/* aliases for screen resolutions */
#define RES_640_480		0
#define RES_800_600		1
#define RES_1024_768		2

/* number of alternate paths to check for textures */
#define AS_NUM_ALT_PATHS 3

/* in-game structs */

/* a struct that defines a vector */
struct vector_t {
	GLdouble x;
	GLdouble y;
	GLdouble z;
	GLdouble w;
};
typedef struct vector_t vector;

/* a struct that defines a quaternion */
struct quaternion_t {
	GLdouble s;
	vector v;
};
typedef struct quaternion_t quaternion;

/* a struct that defines the state of the game */
struct state_t {
	// fullscreen flag
	GLint fullscreen;
	// game paused flag
	GLint pause;
	// game score
	char score[8];
	// third person view flag
	//GLint third_person;
	// string to de displayed
	char status_string[40];
	// current resolution mode
	GLint curr_res;
	// hi definition flag
	GLint hi_res;
	// window width and height in pixels
	GLint windowx;
	GLint windowy;
	// window origin in screen coordinates
	GLint window_xpos;
	GLint window_ypos;
	// mouse cursor coordinates in the window
	GLint mouse_mode;
	GLint mouse_posx;
	GLint mouse_posy;
	GLdouble xprop;
	GLdouble yprop;
	GLint prev_xdiff;
	GLint prev_ydiff;
	// lights
	GLint light0;
	GLint light1;
	// cannons
	GLint left_cannon_fired;
	GLint cannons_linked;
	// shooter game state and flags
	GLint shooter;
	GLint lines;
	GLint trajectories;
	// jetpack state
	GLint jetpack;
	// planet rotation angle
	GLdouble pl_ang;
	// keyboard
	char keypress[8];
	char specialkey[6];
};
typedef struct state_t state_t;

extern state_t *state;
extern int debug;

#endif
