/*------------------------------------------------------------------------------
* Alpha Shooter
* A 3D OpenGL First-Person Shooter game
* Copyright (C) 2005, 2006, 2007 Nicola Cocchiaro <ncocchiaro@users.sf.net>
* Alpha Shooter can be found at http://www.sf.net/projects/alphashooter.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 or, (at your option) any later
* version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*-------------------------------------------------------------------------------
*/


// fps.c
// framerate calculator and displayer

#include <stdio.h>
#include <stdlib.h>
#include "fps.h"

#ifdef WIN32
#include <errno.h>

// gettimeofday replacement for Win32
int gettimeofday (struct timeval *tv, void *tz)
{
	FILETIME ft;
	unsigned long long time;
	
	if (!tv)
	{
		errno = EINVAL;
		return (-1);
	}
	
	GetSystemTimeAsFileTime(&ft);
	// get time and convert from Windows epoch to UNIX epoch
	time = (unsigned long long)((((((unsigned long long)ft.dwHighDateTime) << 32) | (((unsigned long long)ft.dwLowDateTime))) / 10) - 11644473600000000ULL);
	tv->tv_sec = (long)(time / 1000000L);
	tv->tv_usec = (long)(time % 1000000L);
	return (0);
}
#endif

void frame_counter()
{
	GLdouble frame_time;
	
	gettimeofday(fps->fps_end, NULL);
	
	frame_time = fps->fps_end->tv_sec - fps->fps_start->tv_sec +
		((fps->fps_end->tv_usec - fps->fps_start->tv_usec)/1.0E6);
	fps->fps = 1.0/frame_time;
	fps->fps_start->tv_sec = fps->fps_end->tv_sec;
	fps->fps_start->tv_usec = fps->fps_end->tv_usec;
	if (fps->fps > 1.0)
	{
		fps->shift = STEP*100.0 / fps->fps;
		fps->rot = ROTSTEP*100.0 / fps->fps;
	}
	else
	{
		fps->shift = STEP;
		fps->rot = ROTSTEP;
	}
	fps->elapsed += frame_time;
	if (fps->elapsed > 0.5)
	{
		fps->elapsed = 0;
		sprintf(fps->fps_string, "FPS: %2.0f", fps->fps);
	}
}
